@@ -12,6 +12,8 @@
 #define MNTTYPE_REISER		"reiserfs"	/* Reiser file system */
 #define MNTTYPE_XFS		"xfs"	/* SGI XFS file system */
 #define MNTTYPE_AUTOFS		"autofs"	/* Automount mountpoint */
+#define MNTTYPE_JFS		"jfs"	/* JFS file system */
+#define MNTTYPE_NFS4		"nfs4"	/* NFSv4 filesystem */
 
 /* mount options */
 #define MNTOPT_NOQUOTA		"noquota"	/* don't enforce quota */
@@ -23,5 +25,6 @@
 #define MNTOPT_RSQUASH		"rsquash"	/* root as ordinary user */
 #define MNTOPT_BIND		"bind"		/* binded mount */
 #define MNTOPT_LOOP		"loop"		/* loopback mount */
+#define MNTOPT_JQFMT		"jqfmt"		/* journaled quota format */
 
 #endif
