@@ -8,7 +8,7 @@
  *	New quota format implementation - Jan Kara <jack@suse.cz> - Sponsored by SuSE CR
  */
 
-#ident "$Id: quotacheck.c,v 1.40 2004/03/15 14:30:25 jkar8572 Exp $"
+#ident "$Id: quotacheck.c,v 1.47 2005/06/24 14:04:23 jkar8572 Exp $"
 
 #include <dirent.h>
 #include <stdio.h>
@@ -59,6 +59,7 @@ struct dirs {
 };
 
 #define BITS_SIZE 4		/* sizeof(bits) == 5 */
+#define BLIT_RATIO 10		/* Blit in just 1/10 of blit() calls */
 
 dev_t cur_dev;			/* Device we are working on */
 int files_done, dirs_done;
@@ -123,7 +124,7 @@ static int store_dlinks(int type, ino_t i_num)
 	struct dlinks *lptr;
 	uint hash = hash_ino(i_num);
 
-	debug(FL_DEBUG, _("Adding hardlink for ino %d\n"), i_num);
+	debug(FL_DEBUG, _("Adding hardlink for ino %llu\n"), (unsigned long long)i_num);
 
 	for (lptr = links_hash[type][hash]; lptr; lptr = lptr->next)
 		if (lptr->i_num == i_num)
@@ -267,16 +268,24 @@ static loff_t getqsize(char *fname, struct stat *st)
 /*
  * Show a blitting cursor as means of visual progress indicator.
  */
-static inline void blit(void)
+static inline void blit(char *msg)
 {
-	static short bitc = 0;
+	static int bitc = 0;
 	static const char bits[] = "|/-\\";
+	static int slow_down;
 
-	putc(bits[bitc], stdout);
-	putc('\b', stdout);
-	fflush(stdout);
-	bitc++;
-	bitc %= BITS_SIZE;
+	if (flags & FL_VERYVERBOSE && msg) {
+		putchar('\r');
+		printf("%-70s ", msg);
+	}
+	if (flags & FL_VERYVERBOSE || ++slow_down >= BLIT_RATIO) {
+		putchar(bits[bitc]);
+		putchar('\b');
+		fflush(stdout);
+		bitc++;
+		bitc %= BITS_SIZE;
+		slow_down = 0;
+	}
 }
 
 static void usage(void)
@@ -306,7 +315,10 @@ static void parse_options(int argcnt, char **argstr)
 			  setlinebuf(stderr);
 			  break;
 		  case 'v':
-			  flags |= FL_VERBOSE;
+			  if (flags & FL_VERBOSE)
+				flags |= FL_VERYVERBOSE;
+			  else
+				flags |= FL_VERBOSE;
 			  break;
 		  case 'f':
 			  flags |= FL_FORCE;
@@ -404,7 +416,7 @@ static int ext2_direct_scan(char *device)
 		if (inode.i_links_count) {
 			debug(FL_DEBUG, _("Found i_num %ld, blocks %ld\n"), (long)i_num, (long)inode.i_blocks);
 			if (flags & FL_VERBOSE)
-				blit();
+				blit(NULL);
 			uid = inode.i_uid | (inode.i_uid_high << 16);
 			gid = inode.i_gid | (inode.i_gid_high << 16);
 			if (inode.i_uid_high | inode.i_gid_high)
@@ -463,11 +475,13 @@ static int scan_dir(char *pathname)
 		die(2, _("\nCan open directory %s: %s\n"), pathname, strerror(errno));
 
 	chdir(pathname);
+	if (flags & FL_VERYVERBOSE)
+		blit(pathname);
 	while ((de = readdir(dp)) != (struct dirent *)NULL) {
 		if (!strcmp(de->d_name, ".") || !strcmp(de->d_name, ".."))
 			continue;
 		if (flags & FL_VERBOSE)
-			blit();
+			blit(NULL);
 
 		if ((lstat(de->d_name, &st)) == -1) {
 			errstr(_("lstat Cannot stat `%s/%s': %s\nGuess you'd better run fsck first !\nexiting...\n"),
@@ -545,8 +559,8 @@ int ask_yn(char *q, int def)
 	printf("%s [%c]: ", q, def ? 'y' : 'n');
 	fflush(stdout);
 	while (1) {
-		fgets(a, sizeof(a), stdin);
-		if (*a == '\n')
+		fgets(a, sizeof(a)-1, stdin);
+		if (a[0] == '\n')
 			return def;
 		if (!strcasecmp(a, "y\n"))
 			return 1;
@@ -637,7 +651,7 @@ static int rename_files(struct mntent *mnt, int type)
 	int fd;
 #endif
 
-	debug(FL_DEBUG, _("Data dumped.\n"));
+	debug(FL_DEBUG, _("Renaming new files to proper names.\n"));
 	if (get_qf_name(mnt, type, (1 << cfmt), 0, &filename) < 0)
 		die(2, _("Cannot get name of old quotafile on %s.\n"), mnt->mnt_dir);
 	if (stat(filename, &st) < 0) {	/* File doesn't exist? */
@@ -747,8 +761,7 @@ static int dump_to_file(struct mntent *mnt, int type)
 		errstr(_("Cannot finish IO on new quotafile: %s\n"), strerror(errno));
 		return -1;
 	}
-	if (rename_files(mnt, type) < 0)
-	        return -1;
+	debug(FL_DEBUG, _("Data dumped.\n"));
 	if (cfmt == kern_quota_on(mnt->mnt_fsname, type, 1 << cfmt)) {	/* Quota turned on? */
 		char *filename;
 
@@ -762,6 +775,9 @@ static int dump_to_file(struct mntent *mnt, int type)
 			else {
 				int ret;
 
+				/* Rename files - if it fails we cannot do anything better then just turn on quotas again */
+				rename_files(mnt, type);
+
 				if (kernel_iface == IFACE_GENERIC)
 					ret = quotactl(QCMD(Q_QUOTAON, type), mnt->mnt_fsname, util2kernfmt(cfmt), filename);
 				else
@@ -773,6 +789,9 @@ static int dump_to_file(struct mntent *mnt, int type)
 			free(filename);
 		}
 	}
+	else
+		if (rename_files(mnt, type) < 0)
+		        return -1;
 	return 0;
 }
 
@@ -819,9 +838,9 @@ static void check_dir(struct mntent *mnt)
 	int remounted = 0;
 
 	if (lstat(mnt->mnt_dir, &st) < 0)
-		die(2, _("Cannot stat mountpoint %s: %s\n"), mnt, strerror(errno));
+		die(2, _("Cannot stat mountpoint %s: %s\n"), mnt->mnt_dir, strerror(errno));
 	if (!S_ISDIR(st.st_mode))
-		die(2, _("Mountpoint %s isn't directory?!\n"), mnt);
+		die(2, _("Mountpoint %s isn't a directory?!\n"), mnt->mnt_dir);
 	cur_dev = st.st_dev;
 	files_done = dirs_done = 0;
 	if (ucheck)
@@ -864,6 +883,8 @@ Please stop all programs writing to filesystem or use -m flag to force checking.
 #else
 	if (mnt->mnt_dir) {
 #endif
+		if (flags & FL_VERYVERBOSE)
+			putchar('\n');
 		if (scan_dir(mnt->mnt_dir) < 0)
 			goto out;
 	}
@@ -877,7 +898,7 @@ Please stop all programs writing to filesystem or use -m flag to force checking.
 		sub_quota_file(mnt, GRPQUOTA, GRPQUOTA);
 	}
 	if (flags & FL_VERBOSE)
-		fputs(_("done\n"), stderr);
+		fputs(_("done\n"), stdout);
 	debug(FL_DEBUG | FL_VERBOSE, _("Checked %d directories and %d files\n"), dirs_done,
 	      files_done);
 	if (remounted) {
@@ -899,13 +920,14 @@ static int detect_filename_format(struct mntent *mnt, int type)
 	char *option;
 	struct stat statbuf;
 	char namebuf[PATH_MAX];
+	int journal = 0;
 
 	if (type == USRQUOTA) {
 		if ((option = hasmntopt(mnt, MNTOPT_USRQUOTA)))
 			option += strlen(MNTOPT_USRQUOTA);
 		else if (hasmntopt(mnt, MNTOPT_USRJQUOTA)) {
-			errstr(_("Cannot detect quota format for journalled quota on %s\n"), mnt->mnt_dir);
-			return -1;
+			journal = 1;
+			option += strlen(MNTOPT_USRJQUOTA);
 		}
 		else if ((option = hasmntopt(mnt, MNTOPT_QUOTA)))
 			option += strlen(MNTOPT_QUOTA);
@@ -914,13 +936,35 @@ static int detect_filename_format(struct mntent *mnt, int type)
 		if ((option = hasmntopt(mnt, MNTOPT_GRPQUOTA)))
 			option += strlen(MNTOPT_GRPQUOTA);
 		else if (hasmntopt(mnt, MNTOPT_GRPJQUOTA)) {
-			errstr(_("Cannot detect quota format for journalled quota on %s\n"), mnt->mnt_dir);
-			return -1;
+			journal = 1;
+			option += strlen(MNTOPT_GRPJQUOTA);
 		}
 	}
 	if (!option)
 		die(2, _("Cannot find quota option on filesystem %s with quotas!\n"), mnt->mnt_dir);
-	if (*option == '=')	/* If the file name is specified we can't detect quota format from it... */
+	if (journal) {
+		int fmt;
+		char fmtbuf[64], *space;
+		
+		if (!(option = hasmntopt(mnt, MNTOPT_JQFMT))) {
+jquota_err:
+			errstr(_("Cannot detect quota format for journalled quota on %s\n"), mnt->mnt_dir);
+			return -1;
+		}
+		option += strlen(MNTOPT_JQFMT);
+		if (*option != '=')
+			goto jquota_err;
+		space = strchr(option, ',');
+		if (!space)
+			space = option + strlen(option);
+		if (space-option > sizeof(fmtbuf))
+			goto jquota_err;
+		sstrncpy(fmtbuf, option+1, space-option+1);
+		if ((fmt = name2fmt(fmtbuf)) == QF_ERROR)
+			goto jquota_err;
+		return fmt;
+	}
+	else if (*option == '=')	/* If the file name is specified we can't detect quota format from it... */
 		return -1;
 	snprintf(namebuf, PATH_MAX, "%s/%s.%s", mnt->mnt_dir, basenames[QF_VFSV0], extensions[type]);
 	if (!stat(namebuf, &statbuf))
@@ -948,7 +992,7 @@ static void check_all(void)
 	while ((mnt = get_next_mount())) {
 		if (flags & FL_ALL && flags & FL_NOROOT && !strcmp(mnt->mnt_dir, "/"))
 			continue;
-		if (!strcmp(mnt->mnt_type, MNTTYPE_XFS) || !strcmp(mnt->mnt_type, MNTTYPE_NFS)) {
+		if (!strcmp(mnt->mnt_type, MNTTYPE_XFS) || nfs_fstype(mnt->mnt_type)) {
 			debug(FL_DEBUG | FL_VERBOSE, _("Skipping %s [%s]\n"), mnt->mnt_fsname, mnt->mnt_dir);
 			continue;
 		}
