@@ -545,7 +545,8 @@ static void remove_tree(struct quota_handle *h, struct dquot *dquot, uint * blk,
 
 		ref[V2_GETIDINDEX(dquot->dq_id, depth)] = __cpu_to_le32(0);
 		for (i = 0; i < V2_DQBLKSIZE && !buf[i]; i++);	/* Block got empty? */
-		if (i == V2_DQBLKSIZE) {
+		/* Don't put the root block into the free block list */
+		if (i == V2_DQBLKSIZE && *blk != V2_DQTREEOFF) {
 			put_free_dqblk(h, buf, *blk);
 			*blk = 0;
 		}
@@ -748,6 +749,12 @@ static int report_block(struct dquot *dquot, uint blk, char *bitmap,
 	return entries;
 }
 
+static void check_reference(struct quota_handle *h, uint blk)
+{
+	if (blk >= h->qh_info.u.v2_mdqi.dqi_blocks)
+		die(2, _("Illegal reference in %s quota file on %s. Quota file is probably corrupted.\nPlease run quotacheck(8) and try again.\n"), type2name(h->qh_type), h->qh_quotadev);
+}
+
 static int report_tree(struct dquot *dquot, uint blk, int depth, char *bitmap,
 		       int (*process_dquot) (struct dquot *, char *))
 {
@@ -759,15 +766,18 @@ static int report_tree(struct dquot *dquot, uint blk, int depth, char *bitmap,
 	if (depth == V2_DQTREEDEPTH - 1) {
 		for (i = 0; i < V2_DQBLKSIZE >> 2; i++) {
 			blk = __le32_to_cpu(ref[i]);
+			check_reference(dquot->dq_h, blk);
 			if (blk && !get_bit(bitmap, blk))
 				entries += report_block(dquot, blk, bitmap, process_dquot);
 		}
 	}
 	else {
 		for (i = 0; i < V2_DQBLKSIZE >> 2; i++)
-			if ((blk = __le32_to_cpu(ref[i])))
+			if ((blk = __le32_to_cpu(ref[i]))) {
+				check_reference(dquot->dq_h, blk);
 				entries +=
 					report_tree(dquot, blk, depth + 1, bitmap, process_dquot);
+			}
 	}
 	freedqbuf(buf);
 	return entries;
