@@ -38,6 +38,14 @@ static char extensions[MAXQUOTAS + 2][20] = INITQFNAMES;
 static char *basenames[] = INITQFBASENAMES;
 static char *fmtnames[] = INITQFMTNAMES;
 
+/*
+ *	Check for various kinds of NFS filesystem
+ */
+int nfs_fstype(char *type)
+{
+	return !strcmp(type, MNTTYPE_NFS) || !strcmp(type, MNTTYPE_NFS4);
+}
+
 /*
  *	Check whether give filesystem type is supported
  */
@@ -53,12 +61,14 @@ static int correct_fstype(char *type)
 			*next = 0;
 		if (!strcmp(type, MNTTYPE_EXT2) ||
 		    !strcmp(type, MNTTYPE_EXT3) ||
+		    !strcmp(type, MNTTYPE_JFS) ||
 		    !strcmp(type, MNTTYPE_MINIX) ||
 		    !strcmp(type, MNTTYPE_UFS) ||
 		    !strcmp(type, MNTTYPE_UDF) ||
 		    !strcmp(type, MNTTYPE_REISER) ||
 		    !strcmp(type, MNTTYPE_XFS) ||
-		    !strcmp(type, MNTTYPE_NFS)) {
+		    !strcmp(type, MNTTYPE_NFS) ||
+		    !strcmp(type, MNTTYPE_NFS4)) {
 			free(mtype);
 			return 1;
 		}
@@ -437,7 +447,7 @@ int hasquota(struct mntent *mnt, int type)
 	
 	if (!strcmp(mnt->mnt_type, MNTTYPE_XFS))
 		return hasxfsquota(mnt, type);
-	if (!strcmp(mnt->mnt_type, MNTTYPE_NFS))	/* NFS always has quota or better there is no good way how to detect it */
+	if (nfs_fstype(mnt->mnt_type))	/* NFS always has quota or better there is no good way how to detect it */
 		return 1;
 
 	if ((type == USRQUOTA) && (hasmntopt(mnt, MNTOPT_USRQUOTA) || hasmntoptarg(mnt, MNTOPT_USRJQUOTA)))
@@ -541,6 +551,8 @@ int get_qf_name(struct mntent *mnt, int type, int fmt, int flags, char **filenam
 	return -1;
 }
 
+#define START_MNT_POINTS 256	/* The number of mount points we start with... */
+
 /*
  *	Create NULL terminated list of quotafile handles from given list of mountpoints
  *	List of zero length means scan all entries in /etc/mtab
@@ -550,25 +562,30 @@ struct quota_handle **create_handle_list(int count, char **mntpoints, int type,
 {
 	struct mntent *mnt;
 	int gotmnt = 0;
-	static struct quota_handle *hlist[MAXMNTPOINTS];
+	static int hlist_allocated = 0;
+	static struct quota_handle **hlist = NULL;
+
+	if (!hlist_allocated) {
+		hlist = smalloc(START_MNT_POINTS * sizeof(struct quota_handle *));
+		hlist_allocated = START_MNT_POINTS;
+	}
 
 	if (init_mounts_scan(count, mntpoints, mntflags) < 0)
 		die(2, _("Can't initialize mountpoint scan.\n"));
 	while ((mnt = get_next_mount())) {
-		if (strcmp(mnt->mnt_type, MNTTYPE_NFS)) {	/* No NFS? */
-			if (gotmnt+1 == MAXMNTPOINTS)
-				die(2, _("Too many mountpoints with quota. Contact %s\n"), MY_EMAIL);
+		if (!nfs_fstype(mnt->mnt_type)) {	/* No NFS? */
+add_entry:
+			if (gotmnt+1 >= hlist_allocated) {
+				hlist_allocated += START_MNT_POINTS;
+				hlist = srealloc(hlist, hlist_allocated * sizeof(struct quota_handle *));
+			}
 			if (!(hlist[gotmnt] = init_io(mnt, type, fmt, ioflags)))
 				continue;
 			gotmnt++;
 		}
 		else if (!(ioflags & IOI_LOCALONLY) && (fmt == -1 || fmt == QF_RPC)) {	/* Use NFS? */
 #ifdef RPC
-			if (gotmnt+1 == MAXMNTPOINTS)
-				die(2, _("Too many mountpoints with quota. Contact %s\n"), MY_EMAIL);
-			if (!(hlist[gotmnt] = init_io(mnt, type, fmt, ioflags)))
-				continue;
-			gotmnt++;
+			goto add_entry;
 #endif
 		}
 	}
@@ -585,12 +602,15 @@ struct quota_handle **create_handle_list(int count, char **mntpoints, int type,
 int dispose_handle_list(struct quota_handle **hlist)
 {
 	int i;
+	int ret = 0;
 
 	for (i = 0; hlist[i]; i++)
-		if (end_io(hlist[i]) < 0)
+		if (end_io(hlist[i]) < 0) {
 			errstr(_("Error while releasing file on %s\n"),
 				hlist[i]->qh_quotadev);
-	return 0;
+			ret = -1;
+		}
+	return ret;
 }
 
 /*
@@ -866,7 +886,7 @@ static int cache_mnt_table(int flags)
 			continue;
 		}
 
-		if (strcmp(mnt->mnt_type, MNTTYPE_NFS)) {
+		if (!nfs_fstype(mnt->mnt_type)) {
 			if (stat(devname, &st) < 0) {	/* Can't stat mounted device? */
 				errstr(_("Can't stat() mounted device %s: %s\n"), devname, strerror(errno));
 				free((char *)devname);
@@ -919,13 +939,13 @@ static int cache_mnt_table(int flags)
 			for (i = 0; i < mnt_entries_cnt && mnt_entries[i].me_dev != dev; i++);
 		}
 		/* Cope with network filesystems or new mountpoint */
-		if (!strcmp(mnt->mnt_type, MNTTYPE_NFS) || i == mnt_entries_cnt) {
+		if (nfs_fstype(mnt->mnt_type) || i == mnt_entries_cnt) {
 			if (stat(mnt->mnt_dir, &st) < 0) {	/* Can't stat mountpoint? We have better ignore it... */
 				errstr(_("Can't stat() mountpoint %s: %s\n"), mnt->mnt_dir, strerror(errno));
 				free((char *)devname);
 				continue;
 			}
-			if (!strcmp(mnt->mnt_type, MNTTYPE_NFS)) {
+			if (nfs_fstype(mnt->mnt_type)) {
 				/* For network filesystems we must get device from root */
 				dev = st.st_dev;
 				for (i = 0; i < mnt_entries_cnt && mnt_entries[i].me_dev != dev; i++);
@@ -977,10 +997,25 @@ static int process_dirs(int dcnt, char **dirs, int flags)
 	if (dcnt) {
 		check_dirs = smalloc(sizeof(struct searched_dir) * dcnt);
 		for (i = 0; i < dcnt; i++) {
-			if (stat(dirs[i], &st) < 0) {
-				errstr(_("Can't stat() given mountpoint %s: %s\nSkipping...\n"), dirs[i], strerror(errno));
-				continue;
+			if (!strncmp(dirs[i], "UUID=", 5) || !strncmp(dirs[i], "LABEL=", 6)) {
+				char *devname = (char *)get_device_name(dirs[i]);
+
+				if (!devname) {
+					errstr(_("Can't find a device with %s.\nSkipping...\n"), dirs[i]);
+					continue;
+				}
+				if (stat(devname, &st) < 0) {
+					errstr(_("Can't stat() a mountpoint with %s: %s\nSkipping...\n"), dirs[i], strerror(errno));
+					free(devname);
+					continue;
+				}
+				free(devname);
 			}
+			else
+				if (stat(dirs[i], &st) < 0) {
+					errstr(_("Can't stat() given mountpoint %s: %s\nSkipping...\n"), dirs[i], strerror(errno));
+					continue;
+				}
 			check_dirs[check_dirs_cnt].sd_dir = S_ISDIR(st.st_mode);
 			if (S_ISDIR(st.st_mode)) {
 				const char *realmnt = dirs[i];
@@ -988,7 +1023,7 @@ static int process_dirs(int dcnt, char **dirs, int flags)
 				/* Return st of mountpoint of dir in st.. */
 				if (flags & MS_NO_MNTPOINT && !(realmnt = find_dir_mntpoint(&st))) {
 					if (!(flags & MS_QUIET))
-						errstr(_("Can't find filesystem mountpoint for directory %s\n"), dirs[i]);
+						errstr(_("Can't find a filesystem mountpoint for directory %s\n"), dirs[i]);
 					continue;
 				}
 				check_dirs[check_dirs_cnt].sd_dev = st.st_dev;
