@@ -14,7 +14,6 @@
 #define MAXNAMELEN 64		/* Maximal length of user/group name */
 #define MAXTIMELEN 40		/* Maximal length of time string */
 #define MAXNUMLEN 32		/* Maximal length of number */
-#define MAXMNTPOINTS 256	/* Maximal number of mountpoints with quota */
 
 /* Flags for formatting time */
 #define TF_ROUND 0x1		/* Should be printed time rounded? */
@@ -31,12 +30,18 @@
 #define IFACE_VFSV0 2
 #define IFACE_GENERIC 3
 
+/* Path to export table of NFS daemon */
+#define NFSD_XTAB_PATH "/var/lib/nfs/etab"
+
 /* Kernel quota format and supported interface */
 extern int kernel_formats, kernel_iface;
 
 /*
  *	Exported functions
  */
+/* Check whether type is one of the NFS filesystems */
+int nfs_fstype(char *);
+
 /* Convert quota type to written form */
 char *type2name(int);
 
